module functions

use mkl95_lapack

implicit none

contains


function sqr(x)
    doubleprecision :: x,sqr;
    sqr=x**2.0;
endfunction

function gamma(x)
    integer :: cs;
    real :: gamma,x;
    doubleprecision :: pi=3.1415926535897932384626433832795d0

    cs=int(2*x);
    select case(cs)
        case(1)
            gamma=dsqrt(pi);
        case(2)
            gamma=1.0;
        case(3)
            gamma=dsqrt(pi)/2;
        case(4)
            gamma=1.0;
        case(5)
            gamma=3*dsqrt(pi)/4;
        case(6)
            gamma=2.0;
        case(7)
            gamma=15*dsqrt(pi)/8;
        case(8)
            gamma=6.0;
        case(9)
            gamma=105*dsqrt(pi)/16;
        case(10)
            gamma=24.0;
        case (11)
            gamma = 945.0d0*dsqrt(pi)/32.0d0
        case (12)
            gamma = 120.0d0
        case (13)
            gamma = 10395*dsqrt(pi)/64.0d0
        case (14)
            gamma = 720.0d0
        case (15)
            gamma = 135135.0d0*dsqrt(pi)/128.0d0
        case (16)
            gamma = 5040.0d0
        case (17)
            gamma = 2027025*dsqrt(pi)/256.0d0
        case (18)
            gamma = 40320.0d0
        case (19)
            gamma = 34459425*dsqrt(pi)/512.0d0
        case (20)
            gamma = 362880.0d0 
        endselect   
endfunction

subroutine DistributeElectrons(elecs, numelecs)
    integer :: elecs(103,4),numelecs;
    integer :: n,l,ml,ms,i;
    

    n=1;l=0;ml=0;ms=1;
    do i=1,numelecs
        elecs(i,1)=n; elecs(i,2)=l; elecs(i,3)=ml; elecs(i,4)=ms;
        if(ml==l)then
            if(ms==1)then
                ms=-1;
                ml=-l;
            else
                if(l-1 >= 0)then
                    n=n+1;
                    l=l-1;
                    ml=-l;
                    ms=1;
                else
                    if(n==1)then
                        n=2;
                    elseif(n==2)then
                        l=1;
                    elseif(n==3)then
                        l=1;
                    elseif(n==4)then
                        n=3;l=2;
                    elseif(n==5)then
                        n=4;l=2;
                    elseif(n==6)then
                        n=4;l=3;
                    elseif(n==7)then
                        n=5;l=3;
                    elseif(n==8)then
                        n=5;l=4;
                    endif
                    ms=1;
                    ml=-l;                    
                endif
            endif
        else
            ml=ml+1;
        endif        
    enddo
endsubroutine

function GetMaxn(elecs,numelecs)
    integer, intent(in) :: elecs(103,4),numelecs;
    integer :: GetMaxn;
    integer :: i;
    GetMaxn=1;
    do i=1,numelecs
        GetMaxn = max(GetMaxn,elecs(i,1));
    enddo
endfunction

function GetMaxl(elecs,numelecs)
    integer, intent(in) :: elecs(103,4),numelecs;
    integer :: GetMaxl;
    integer :: i;
    GetMaxl=0;
    do i=1,numelecs
        GetMaxl = max(GetMaxl,elecs(i,2));
    enddo
endfunction

function CalcHartreePot(r, ed, edsteps, edpts, divs)
    doubleprecision :: CalcHartreePot;
    doubleprecision :: r;
    doubleprecision,allocatable :: edsteps(:),ed(:);
    integer,allocatable :: edpts(:);
    integer :: divs;

    integer :: i,tot,lim,bank,next;
    doubleprecision :: pot,rp,drp; !rp: r prime, drp: d(r prime), a&b: integration limits
    
    rp=0; pot=0; drp=0; tot=0; lim=0;
    
    do i=1,divs
        tot = tot+edpts(i);
    enddo    
    
    do i=1,divs
        if( (r-edsteps(i)) > 0 )then
            lim = lim+edpts(i);
        elseif(r-edsteps(i) <= 0)then
            lim = lim + floor(edpts(i)/(edsteps(i)-edsteps(i-1))*(r-edsteps(i-1))) + 1; !+1  since ed is 1-based array
            exit;
        endif
    enddo

    bank=1;
    next = edpts(1);
    do i=1,tot
        if(i>next)then
            bank=bank+1;
            next = next+edpts(bank);
        endif
        
        drp=(edsteps(bank)-edsteps(bank-1))/edpts(bank);        
        
        ! dont do this if i is equal to 1 in order to avoid singularities.
        if(i/=1)then
            if(i<lim .and. r>0)then
                pot = pot + ed(i)/r*drp;
            elseif(i>=lim)then
                pot = pot + ed(i)/rp*drp;
            endif
        endif
        rp=rp+drp;
    enddo        

    CalcHartreePot = pot;
endfunction

function CalcHartreePotDer(r, ed, edsteps, edpts, divs)
    doubleprecision :: CalcHartreePotDer;
    doubleprecision :: r;
    doubleprecision,allocatable :: ed(:),edsteps(:);
    integer :: divs;
    integer,allocatable :: edpts(:);
    
    integer :: i,bank,next,lim;
    doubleprecision :: potd,rp,drp;
    
    potd=0; rp=0; drp=0; lim=0;  

    do i=1,divs
        if( (r-edsteps(i)) < 0 )then
            lim = lim+edpts(i);
        elseif(r-edsteps(i) < 0)then
            lim = lim + floor(edpts(i)/(edsteps(i)-edsteps(i-1))*(r-edsteps(i-1))) + 1; !+1  since ed is 1-based array
            exit;
        endif
    enddo
  
    bank=1;
    next = edpts(1);
    do i=1,lim
        if(i>next)then
            bank=bank+1;
            next = next+edpts(bank);
        endif
        
        drp = (edsteps(bank)-edsteps(bank-1))/edpts(bank);
              
        if(i/=1)then
            potd = potd - 1/r**2*ed(i)*drp;
        endif
        rp=rp+drp;
    enddo
    
    CalcHartreePotDer = potd;
endfunction

!remove the outermost electron of spin ms
subroutine IonizeAtom(elecs, numelecs, ms)
    integer :: elecs(103,4), numelecs, ms;

    integer :: i,j, nMaxn,nMaxl;
    
    nMaxn = GetMaxn(elecs,numelecs);
    nMaxl = GetMaxl(elecs,numelecs);
    do i=1,numelecs
        if(elecs(i,1)==nMaxn .and. elecs(i,2)==nMaxl .and. elecs(i,4)==ms)then
            exit;!we only remove one electron, i is now the position of the electron to be removed
        endif
    enddo
    
    if(i<numelecs)then
        elecs(i:(numelecs-1),1)=elecs((i+1):numelecs,1);
        elecs(i:(numelecs-1),2)=elecs((i+1):numelecs,2);
        elecs(i:(numelecs-1),3)=elecs((i+1):numelecs,3);
        elecs(i:(numelecs-1),4)=elecs((i+1):numelecs,4);
    endif
    numelecs=numelecs-1;
endsubroutine

!remove electron
subroutine RemoveElectron(elecs, numelecs, n, l, ms)
    integer :: elecs(103,4), numelecs, n, l, ms;
    
    integer :: i;
    
    do i=1,numelecs
        if(elecs(i,1)==n .and. elecs(i,2)==l .and. elecs(i,4)==ms)then
            exit;!we only remove one electron, i is now the position of the electron to be removed
        endif
    enddo
 
    if(i<numelecs)then
        elecs(i:(numelecs-1),1)=elecs((i+1):numelecs,1);
        elecs(i:(numelecs-1),2)=elecs((i+1):numelecs,2);
        elecs(i:(numelecs-1),3)=elecs((i+1):numelecs,3);
        elecs(i:(numelecs-1),4)=elecs((i+1):numelecs,4);
    endif
    numelecs=numelecs-1;
endsubroutine

endmodule