!=============================
! MAIN PROGRAM STARTS HERE
!=============================

program HF

use mkl95_blas
use mkl95_lapack

use functions
use matelem
use basis
use fileoutput

implicit none

! Eigenvalue/Eigenvector calculation variables
!----------------------------------------
integer, parameter :: TheZ=68;
integer :: MaxZ,MinZ;
integer :: elecs(103,4), basc(0:3);
integer :: Z,nNumElecs;
integer :: nPos,nMatSize,nMaxIters=25,nElecNum;

doubleprecision :: dSumLim=0.0001,dSumNew,dSumOld,dNorm;

doubleprecision :: bas(30,0:3);
doubleprecision, allocatable :: EigVals(:), EigVecs(:,:), Hmat(:,:), Smat(:,:), H0mat(:,:), Vmat(:,:), vectmp(:);

integer :: l,ms;
integer :: iter;

! Counter variables
!----------------------------------------
integer h,i,j,k,m,ndx;

! Electron density and Hartree potential calculation variables
!-------------------------------------------------------------

integer,parameter :: nEDDivs=4;
integer :: nTot;
integer,allocatable :: nEDPts(:);
doubleprecision :: r,dHPot,dHPotd;
doubleprecision, allocatable :: dElecDens(:),dEDSteps(:);


allocate(nEDPts(0:nEDDivs));
allocate(dEDSteps(0:nEDDivs));
nEDPts(0) = 0; nEDPts(1)=100000; nEDPts(2)=75000; nEDPts(3)=30000; nEDPts(4)=20000;
dEDSteps(0)=0.0; dEDSteps(1)=0.125; dEDSteps(2)=0.375; dEDSteps(3)=1.0; dEDSteps(4)=20.0;

!nEDPts(0) = 0; nEDPts(1)=1000; nEDPts(2)=750; nEDPts(3)=300; nEDPts(4)=200;
!dEDSteps(0)=0.0; dEDSteps(1)=0.125; dEDSteps(2)=0.375; dEDSteps(3)=1.0; dEDSteps(4)=20.0;


open(unit=1,file='hfdata1.txt');
open(unit=2,file='hpot.txt');

MinZ=TheZ;
MaxZ=TheZ;
do Z=MinZ,MaxZ
    nNumElecs = Z;
    call LoadBasis(bas,Z);
    call DistributeElectrons(elecs,Z);
    call BasisCount(basc,Z);
   
    do l=1,nNumElecs
            write(1,*) elecs(l,1),elecs(l,2),elecs(l,3),elecs(l,4);
    enddo
    
    write(*,*);
    call RemoveElectron(elecs,nNumElecs,6,0,1);
    Call RemoveElectron(elecs,nNumElecs,6,0,-1);
    !call RemoveElectron(elecs,nNumElecs,4,3,1);
    call RemoveElectron(elecs,nNumElecs,4,3,-1);
    do l=1,nNumElecs
            write(1,*) elecs(l,1),elecs(l,2),elecs(l,3),elecs(l,4);
    enddo
    
    dSumOld=0.0;
    allocate(EigVals(Z),EigVecs(30,Z));
    do iter=1,nMaxIters        
        do nElecNum=1,nNumElecs !electron loop
            nMatSize=basc(elecs(nElecNum,2));
            
            allocate(Hmat(nMatSize,nMatSize),H0mat(nMatSize,nMatSize),Smat(nMatSize,nMatSize),Vmat(nMatSize,nMatSize));
            allocate(vectmp(nMatSize));
            H0mat=0; Smat=0; Vmat=0;
             
            l=elecs(nElecNum,2); ms=elecs(nElecNum,4);           
            call SetupH0mat(H0mat,bas,basc,l,Z);
            call SetupSmat(Smat,bas,basc,l);
            
            if(iter/=1)then            
                call SetupVmat(Vmat,bas,basc,l,ms,elecs,nNumElecs,EigVecs);
            endif
            
            Hmat = H0mat + Vmat;
            vectmp=0;
            
            call sygv(Hmat,Smat,vectmp,1,'V');
            nPos = elecs(nElecNum,1)-elecs(nElecNum,2);
            EigVals(nElecNum)=vectmp(nPos);
            EigVecs(1:nMatSize,nElecNum)=Hmat(:,nPos);
            
            deallocate(Hmat,H0mat,Smat,Vmat);
            deallocate(vectmp);
                                    
        enddo !electron loop
        
        write(*,'(A1, $)') '.';

        !sum all eigenvalues to check for convergence
        dSumNew=0;
        do i=1,nNumElecs
            dSumNew = dSumNew + EigVals(i);
        enddo
        if(abs(dSumOld - dSumNew) > dSumLim)then
            dSumOld = dSumNew;
        else
            exit;
        endif

    enddo !iterations

    write(*,*);
    write(*,*) 'ELECTRON DENSITY CALCULATION';
    write(1,*);
    write(1,*) 'ELECTRON DENSITY DATA BEGIN';
    write(1,*);
       
    
    nTot=0;
    do i=1,nEDDivs
        nTot = nTot+nEDPts(i);
    enddo
    
    allocate(dElecDens(nTot));
    ndx=1;
    do h=1,nEDDivs
        do m=1,nEDPts(h)
            dElecDens(ndx)=0;
            r = dEDSteps(h-1) + (dEDSteps(h)-dEDSteps(h-1))/nEDPts(h)*(m-1);
            do i=1,nNumElecs
                do j=1,basc(elecs(i,2))
                    do k=1,basc(elecs(i,2))
                        !dElecDens(ndx) = dElecDens(ndx) + EigVecs(j,i)*EigVecs(k,i)*r**(2*elecs(i,2)+2)*exp(-r*r*(bas(k,elecs(i,2))+bas(j,elecs(i,2))));
                        dElecDens(ndx) = dElecDens(ndx) + EigVecs(j,i)*EigVecs(k,i)*r**(2*elecs(i,2))*exp(-r*r*(bas(k,elecs(i,2))+bas(j,elecs(i,2))));
                    enddo
                enddo
            enddo
            write(1,'(F11.6, F16.6)') r, dElecDens(ndx);
            ndx=ndx+1;
        enddo
        
    enddo

    write(*,*);
    write(*,*) 'HARTREE POTENTIAL CALCULATION';
    
    dHPot=0; dHPotd=0; r=0;
    do h=1,nEDDivs
        do m=1,nEDPts(h)
            r = dEDSteps(h-1) + (dEDSteps(h)-dEDSteps(h-1))/nEDPts(h)*(m-1);
            
            !dHPot = CalcHartreePot(r,dElecDens,dEDSteps,nEDPts,nEDDivs);
            !dHPotd = CalcHartreePotDer(r,dElecDens,dEDSteps,nEDPts,nEDDivs);
            !write(2,'(F14.7,F14.7,F14.7)') r, dHPot, dHPotd;
            
            if(mod(m,1000)==0)then
                write(*,'(F14.7,F14.7,F14.7)') r, dHPot, dHPotd;
            endif
        enddo
    enddo
    
    write(1,*);
    write(1,'(I2.2, A2, F11.3)') iter,', ',Energy(bas,basc,elecs,nNumElecs,Z,EigVals,EigVecs);
    write(*,*);
    write(*,*) iter,Z,Energy(bas,basc,elecs,nNumElecs,Z,EigVals,EigVecs);

    call OutputEigenVecs(basc,elecs,EigVecs,Z,nNumElecs);
    call OutputBasis(Z,bas,basc);
    
    deallocate(EigVals,EigVecs,dElecDens);
enddo !atoms

close(1);
close(2);

endprogram






!    do m=1,nPoints
!        dHPot=0;
!        r = 10.0/nPoints*(m-1);
!        do i=1,nNumElecs
!            dHPot = dHPot + CalcHartreePotCont(r, bas, basc, elecs, i, EigVecs);
!        enddo
!        write(1,'(F11.3)') dHPot;
!        write(*,*) r;
!    enddo
