module matelem

use functions

implicit none

!doubleprecision arrCoulLookUp(:,:),arrExchLookUp(:,:);

contains

subroutine SetupH0mat(H,bas,basc,l,nuccharge)
    doubleprecision :: bas(30,0:3);
    integer basc(0:3),l,nuccharge;
    
    doubleprecision :: H(:,:);
    integer :: n,m;

    do m=1,basc(l)
        do n=m,basc(l)
            H(m,n) = Hmn(bas,m,n,l,nuccharge);
            H(n,m) = H(m,n);
        enddo
    enddo
endsubroutine

subroutine SetupSmat(S,bas,basc,l)
    doubleprecision :: bas(30,0:3),S(:,:);
    integer :: n,m,l,basc(0:3);
       
    do m=1,basc(l)
        do n=m,basc(l)
            S(m,n) = Smn(bas,m,n,l);
            S(n,m) = S(m,n);
        enddo
    enddo
endsubroutine

subroutine SetupVmat(V,bas,basc,l,ms,elecs,numelecs,eigvecs)
    doubleprecision :: bas(30,0:3),V(:,:),eigvecs(:,:);
    integer :: l,ms,m,n,basc(0:3),elecs(103,4),numelecs;
      
    do m=1,basc(l)
        do n=m,basc(l)
            V(m,n)=Vmn(bas,basc,m,n,l,ms,elecs,numelecs,eigvecs)
            V(n,m)=V(m,n);
        enddo
    enddo
endsubroutine

!bas=basis, basc=basiscount, l=angular momentum length, ms=spin, elecs=electron config,
!nelecs=total number of electrons, eigvecs=eigen vectors
function Vmn(bas,basc,m,n,l,ms,elecs,numelecs,eigvecs)
    doubleprecision :: bas(30,0:3),eigvecs(:,:);
    integer :: m,n,l,ms,numelecs,basc(0:3),elecs(103,4);
    
    doubleprecision :: Vmn;
    integer :: i,j,l2,elecnum;
        
    Vmn=0;
    do elecnum=1,numelecs
        l2=elecs(elecnum,2);
        
        do i=1,basc(l2)
            do j=1,basc(l2)                
                Vmn = Vmn+eigvecs(i,elecnum)*eigvecs(j,elecnum)*CoulombInteraction(bas(m,l),bas(n,l),bas(i,l2),bas(j,l2),l,l2);

                if(elecs(elecnum,4)==ms)then
                    Vmn = Vmn-eigvecs(i,elecnum)*eigvecs(j,elecnum)*ExchangeInteraction(bas(m,l),bas(n,l),bas(i,l2),bas(j,l2),l,l2);
                endif                
            enddo !matrix columns
        enddo !matrix rows
    enddo !electron
endfunction

function Hmn(bas,m,n,l,Z)
    doubleprecision :: bas(30,0:3),Hmn,bm,bn;
    integer :: l,m,n,Z;
    bm=bas(m,l);bn=bas(n,l);
    Hmn = -1.0/(2.0*(bm+bn)**(1.0+l))*Z*gamma(1.0+l)+bm*bn/(bm+bn)**(5.0/2.0+l)*gamma(5.0/2.0+l);
endfunction

function Smn(bas,m,n,l)
    doubleprecision :: bas(30,0:3),Smn,bm,bn;
    integer :: l,m,n;
    bm=bas(m,l);bn=bas(n,l);
    Smn = 1.0/(2.0*(bm+bn)**(3.0/2.0+l))*gamma(3.0/2.0+l);
endfunction

function CoulombInteraction(b1,b2,b3,b4,l1,l2)
    doubleprecision :: b1,b2,b3,b4,p,q,CoulombInteraction;
    integer :: l1,l2;
    
    p=b1+b2;
    q=b3+b4;
    if(l1==0 .and. l2==0)then
        CoulombInteraction = ssss(p,q);
    elseif(l1==0 .and. l2==1)then
        CoulombInteraction = ppss(q,p);
    elseif(l1==1 .and. l2==0)then
        CoulombInteraction = ppss(p,q);
    elseif(l1==1 .and. l2==1)then
        CoulombInteraction = pppp(p,q);
    elseif(l1==0 .and. l2==2)then
        CoulombInteraction = ddss(q,p);
    elseif(l1==2 .and. l2==0)then
        CoulombInteraction = ddss(p,q);
    elseif(l1==1 .and. l2==2)then
        CoulombInteraction = ddpp(q,p);
    elseif(l1==2 .and. l2==1)then
        CoulombInteraction = ddpp(p,q);
    elseif(l1==2 .and. l2==2)then
        CoulombInteraction = dddd(p,q);
    elseif(l1==0 .and. l2==3)then
        CoulombInteraction = ffss(q,p);
    elseif(l1==3 .and. l2==0)then
        CoulombInteraction = ffss(p,q);
    elseif(l1==1 .and. l2==3)then
        CoulombInteraction = ffpp(q,p);
    elseif(l1==3 .and. l2==1)then
        CoulombInteraction = ffpp(p,q);    
    elseif(l1==2 .and. l2==3)then
        CoulombInteraction = ffdd(q,p);
    elseif(l1==3 .and. l2==2)then
        CoulombInteraction = ffdd(p,q);
    elseif(l1==3 .and. l2==3)then
        CoulombInteraction = ffff(p,q);
    endif
endfunction

function ExchangeInteraction(b1,b2,b3,b4,l1,l2)
    doubleprecision :: b1,b2,b3,b4,p,q,ExchangeInteraction;
    integer :: l1,l2;
    
    p=b1+b4;
    q=b3+b2; 
    if(l1==0 .and. l2==0)then
        ExchangeInteraction = ssss(p,q);
    elseif(l1==0 .and. l2==1)then
        ExchangeInteraction = psps(p,q);
    elseif(l1==1 .and. l2==0)then
        ExchangeInteraction = psps(q,p);
    elseif(l1==1 .and. l2==1)then
        ExchangeInteraction = ppppe(p,q);
    elseif((l1==0 .and. l2==2) .or. (l1==2 .and. l2==0))then
        ExchangeInteraction = dsds(p,q);
    elseif((l1==1 .and. l2==2) .or. (l1==2 .and. l2==1))then
        ExchangeInteraction = dpdp(p,q);
    elseif(l1==2 .and. l2==2)then
        ExchangeInteraction = dddde(p,q);
    elseif((l1==0 .and. l2==3) .or. (l1==3 .and. l2==0))then
        ExchangeInteraction = fsfs(p,q);
    elseif((l1==1 .and. l2==3) .or. (l1==3 .and. l2==1))then
        ExchangeInteraction = fpfp(p,q);        
    elseif((l1==2 .and. l2==3) .or. (l1==3 .and. l2==2))then
        ExchangeInteraction = fdfd(p,q);
    elseif(l1==3 .and. l2==3)then
        ExchangeInteraction = ffffe(p,q);
    endif  
endfunction

!subroutine SetupH0mat(H,bas,basc,l,nuccharge)
function Energy(bas,basc,elecs,numelecs,Z,EigVals,EigVecs)
    doubleprecision :: Energy,E0,EigVals(:),EigVecs(:,:),bas(30,0:3);
    doubleprecision, allocatable :: H0mat(:,:);
    integer numelecs,elecs(103,4),basc(0:3),Z;
    integer i,j,k;
    
    Energy=0;
    do k=1,numelecs
        E0=0.0;
        allocate( H0mat( basc(elecs(k,2)),basc(elecs(k,2)) ) );
        call SetupH0mat(H0mat,bas,basc,elecs(k,2),Z);
        do i=1,basc(elecs(k,2))
            do j=1,basc(elecs(k,2))
                E0=E0+EigVecs(i,k)*EigVecs(j,k)*H0mat(i,j);
            enddo
        enddo
        deallocate(H0mat);
        Energy=Energy+E0/2+EigVals(k)/2;
    enddo
endfunction


function ssss(bp,bq)
doubleprecision ::ssss,bp,bq
doubleprecision :: pi = 3.1415926535897932384626433832795d0
ssss = sqrt(pi/(bp+bq))/(8.0*bp*bq);
endfunction ssss

function ppss(bp,bq)
doubleprecision ::ppss,bp,bq
ppss = ssss(bp,bq)*(3.0*bp+2.0*bq)/(2.0*bp*(bp+bq))
endfunction

function psps(bp,bq)
doubleprecision :: psps,bp,bq
psps = ssss(bp,bq)/(2.0*(bp+bq))
endfunction psps

function dsds(bp,bq)
doubleprecision :: dsds,bp,bq
dsds = ssss(bp,bq)*3.0/(4.0*(bp+bq)*(bp+bq))
endfunction dsds

function fsfs(bp,bq)
doubleprecision :: fsfs,bp,bq
fsfs = ssss(bp,bq)*15.0/(8.0*(bp+bq)*(bp+bq)*(bp+bq))
endfunction fsfs

function pppp(bp,bq)
doubleprecision :: pppp,bp,bq
pppp = ssss(bp,bq)*(2.0*bp*bp+2.0*bq*bq+5.0*bp*bq)*3.0/(4.0*bp*bq*(bp+bq)*(bp+bq))
endfunction pppp

function ppppe(bp,bq)
doubleprecision :: ppppe,bp,bq
ppppe = ssss(bp,bq)*(2.0*bp*bp+2.0*bq*bq+7.0*bp*bq)/(4.0*bp*bq*(bp+bq)*(bp+bq))
endfunction ppppe

function ddss(bp,bq)
doubleprecision :: ddss,bp,bq
ddss = ssss(bp,bq)*(15.0*bp*bp+8.0*bq*bq+20.0*bp*bq)/(4.0*bp*bp*(bp+bq)*(bp+bq))
endfunction

function ddpp(bp,bq)
doubleprecision :: bp,bq,ddpp
ddpp = ssss(bp,bq)*(10.0*bp*bp*bp+35.0*bp*bp*bq+28.0*bp*bq*bq+8.0*bq*bq*bq)*3.0/(8.0*bp*bp*bq*(bp+bq)*(bp+bq)*(bp+bq))
endfunction ddpp

function dpdp(bp,bq)
doubleprecision :: bp,bq,dpdp
dpdp = ssss(bp,bq)*(4.0*bp*bp+4.0*bq*bq+23.0*bp*bq)/(8.0*bp*bq*(bp+bq)*(bp+bq)*(bp+bq))
endfunction dpdp

function fpfp(bp,bq)
doubleprecision :: bp,bq,fpfp
fpfp = ssss(bp,bq)*(6.0*bp*bp+6.0*bq*bq+47.0*bp*bq)*3.0/(16.0*bp*bq*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq))
endfunction fpfp

function dddd(bp,bq)
doubleprecision :: bp,bq,dddd
dddd = ssss(bp,bq)*(8.0*(bp*bp*bp*bp+bq*bq*bq*bq)+36.0*(bp*bp*bp*bq+bp*bq*bq*bq)+63.0*bp*bp*bq*bq)*15.0/(16.0*sqr(bp*bq*(bp+bq)*(bp+bq)))
endfunction dddd

function dddde(bp,bq)
doubleprecision :: dddde,bp,bq
dddde = ssss(bp,bq)*(8.0*(bp*bp*bp*bp+bq*bq*bq*bq)+40.0*(bp*bp*bp*bq+bp*bq*bq*bq)+99.0*bp*bp*bq*bq)*3.0/(16.0*sqr(bp*bq*(bp+bq)*(bp+bq)))
endfunction dddde

function fdfd(bp,bq)
doubleprecision :: fdfd,bp,bq
fdfd = ssss(bp,bq)*(8.0*(bp*bp*bp*bp+bq*bq*bq*bq)+52.0*(bp*bp*bp*bq+bp*bq*bq*bq)+193.0*bp*bp*bq*bq)*9.0/(32.0*sqr(bp*bq*(bp+bq)*(bp+bq))*(bp+bq))
endfunction fdfd

function ffss(bp,bq)
doubleprecision :: ffss,bp,bq
  ffss = ssss(bp,bq)*(35.0*bp*bp*bp+70.0*bp*bp*bq+56.0*bp*bq*bq+16.0*bq*bq*bq)*3.0/(8.0*bp*bp*bp*(bp+bq)*(bp+bq)*(bp+bq))
endfunction ffss

function ffpp(bp,bq)
doubleprecision :: bp,bq,ffpp
  ffpp = ssss(bp,bq)*(70.0*bp*bp*bp*bp+315.0*bp*bp*bp*bq+378.0*bp*bp*bq*bq+216.0*bp*bq*bq*bq+48.0*bq*bq*bq*bq)*3.0/(16.0*bp*bp*bp*bq*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq))
endfunction ffpp

function ffdd(bp,bq)
doubleprecision :: bp,bq,ffdd
  ffdd = ssss(bp,bq)*(56.0*bp*bp*bp*bp*bp+308.0*bp*bp*bp*bp*bq+693.0*bp*bp*bp*bq*bq+594.0*bp*bp*bq*bq*bq+264.0*bp*bq*bq*bq*bq+48.0*bq*bq*bq*bq*bq)*15.0/(32.0*bp*bp*bp*bq*bq*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq))
endfunction ffdd

function ffff(bp,bq)
doubleprecision :: ffff,bp,bq
  ffff = ssss(bp,bq)*(16.0*(bp*bp*bp*bp*bp*bp+bq*bq*bq*bq*bq*bq)+104.0*(bp*bp*bp*bp*bp*bq+bp*bq*bq*bq*bq*bq)+286.0*(bp*bp*bp*bp*bq*bq+bp*bp*bq*bq*bq*bq)+429.0*bp*bp*bp*bq*bq*bq)*315.0/(64.0*bp*bp*bp*bq*bq*bq*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq))
endfunction ffff

function ffffe(bp,bq)
doubleprecision :: ffffe,bp,bq
ffffe = ssss(bp,bq)*(80.0*(bp*bp*bp*bp*bp*bp+bq*bq*bq*bq*bq*bq)+552.0*(bp*bp*bp*bp*bp*bq+bp*bq*bq*bq*bq*bq)+1698.0*(bp*bp*bp*bp*bq*bq+bp*bp*bq*bq*bq*bq)+3607.0*bp*bp*bp*bq*bq*bq)*9.0/(64.0*bp*bp*bp*bq*bq*bq*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq)*(bp+bq))
endfunction ffffe


endmodule